#! /bin/sh
# the next line restarts using wish \
exec /usr/bin/wish "$0" "$@"

# A texture editing utility for POV-Ray, written in Tcl/Tk
# Code by m2 (c) 2008
# 
# tkxture.tcl - this main file.
# Version
set version v00.07

# UI built with SpecTcl, a Sun Microsystems utility.
# To simplify, variables used in building the POV-Ray script/include file(s), are global.

# Include the user interface file.
source ./tkxture.ui.tcl
# Include colormap presentation.
source ./tkxColormap.tcl
# Include buffer editor that makes the POV-Ray texture file.
source ./tkxPOVtexture.tcl
# Include extra ui widgets for overlays.
# source ./uiOverlays.ui.tcl

# Title the application.
wm title . "Tkxture $version"

# Provide a frame-work for the ui.
frame .frameMain
# frame .frameMain.frameExtra

# Call the ui file "command" with the name of the container.
tkxture_ui .frameMain
# uiOverlays_ui .frameExtra
# uiOverlays_ui .frameMain.frameExtra

# Pack the ui into the provided container.
pack configure .frameMain -side top -fill y -expand 1

set filePOV "tkxture.inc"
set fileArchive "tkxture.inc"
set bufferTexture "..."
set activeTexture "Temp"
set activeFile "tkxture.inc"
set fileImageMap "some.png"

set varInterpBilinear 1
set varInterpNorm 0
set varImageMapPlanar   1
set varImageMapSphere   0
set varImageMapCylinder 0
set varImageMapTorus    0
set varImageMapOnce     0
set varImageMapOnceTemp 0

set varHiQ 0

set stateColormap 0
set stateGradient 0

set varRed        0.20
set varGreen      0.40
set varBlue       0.60
set varRedHex     0
set varGreenHex   0
set varBlueHex    0
set varColorName "0x000000"

# set varColorDel 0
# set varColorNew 0
set activeColor 0
set varCurrentColor 1
set listColormapRed        { 0.00 1.00 1.00 1.00 1.00 0.00 }
set listColormapGreen      { 0.00 1.00 1.00 0.50 0.00 0.00 }
set listColormapBlue       { 0.00 1.00 0.00 0.00 0.00 0.00 }
set listColormapFilter     { 0.30 0.00 0.00 0.00 0.50 0.30 }
set listColormapTransmit   { 0.25 0.00 0.00 0.00 0.50 0.25 }
set listColormapLevel      { 0.00 0.10 0.15 0.20 0.25 1.00 }
set varLabelColormapN "of 6 colors"
set nColors 6
set nColorsAdded 4
set varEntryColormapN 1

set activePattern "agate"
set stateNoiseGenerator 1
set varNoiseGenerator 2

set varInterior "air"
set varAmbientLock 1

set varTurbulenceLock 1
set varWarpEnable 1
set activeWarp "turbulence"
set varTurbulenceX 0.60 
set varTurbulenceY 0.60
set varTurbulenceZ 0.60

set varWarpBeforeTurb 1
set varWarpAfterTurb 0
set varWarpTurbPriority "before"

set varTranslateBeforeWarp 0
set varTranslateAfterWarp 1
set varTranslateWarpPriority "after"

set varScaleLock  1
set varScaleX     1.00
set varScaleY     1.00
set varScaleZ     1.00
set varScaleX10   0
set varRotationLock 0
set varRotationX 0
set varRotationY 0
set varRotationZ 0



# Default values for pattern modifiers, from POV-Ray docs;
# dist_exp        : 0
# falloff         : 2.0
# major_radius    : 1
# map_type        : 0
# noise_generator : 2
# orientation     : <0,0,1>
# poly_wave       : 1.0
# strength        : 1.0
# turbulence      : <0,0,0>
set varFrequency  1
set varLambda     2.00
set varOctaves    6
set varOmega      0.50
set varPhase      0

set varCrackleForm1 -1
set varCrackleForm2  1
set varCrackleForm3  0

set varCrackleMetric 2
set varCrackleOffset 0.00

set varBlackHoleTurbulenceX 0.00
set varBlackHoleTurbulenceY 0.00
set varBlackHoleTurbulenceZ 0.00
set varBlackHoleLocationX 0.00
set varBlackHoleLocationY 0.00
set varBlackHoleLocationZ 0.00

set varBlackHoleRadius 0.50
set varBlackHoleFalloff 2.00
set varBlackHoleStrength 1.00
set varBlackHoleRepeatX 0.00
set varBlackHoleRepeatY 0.00
set varBlackHoleRepeatZ 0.00

set varCSTOrientationX 0
set varCSTOrientationY 0
set varCSTOrientationZ 1

set varRepeatDirectionX 1
set varRepeatDirectionY 0
set varRepeatDirectionZ 0

set varAmbientRed   0.10
set varAmbientGreen 0.10
set varAmbientBlue  0.10

set varDiffuse 0.60
set varBrilliance 1.00
set varConserveEnergy 0
set varPhongSize 40
set varMetallic 0.00
set varSpecular 0.00
set varRoughness 0.05
set varExponent 1.00

set varReflMetallic 0.00

# Interior
# IORs air 1.0, water is 1.33, glass is 1.5, diamond is 2.4.
# ior                : 1.0
# caustics           : 0.0
# dispersion         : 1.0
# dispersion_samples : 7
# fade_distance      : 0.0 
# fade_power         : 0.0
# fade_color         : <0,0,0>

set varIor                 1.00
set varCaustics            0.00
set varDispersion          1.00
set varDispSamples         7
set varFadeDist            0.00 
set varFadePower           1.00
set varFadeLock            1

# Fractal modifiers for julia and mandelbrot
set varIterations          5
set varFractalExponent     2
set varMagnetType          0 
set varComplexA            0.00 
set varComplexB            0.00
set varBumpSize            0.00

set varExteriorType        1
set varExteriorFactor      1.00
set varInteriorType        0
set varInteriorFactor      1.00

# Setup color_map structure
proc initColormap { } {
   global varRed varGreen varBlue
   global listColormapRed listColormapGreen listColormapBlue 
   global listColormapFilter listColormapTransmit
   global listColormapLevel

   set i 0
   set j 0.00
   set k [ expr 1.00 / 256 ]

   # 255 positions in color_map
   while { $i < 255 } {
   
      set i [ expr $i + 1 ]

      lappend listColormapRed $j
      lappend listColormapGreen $j
      lappend listColormapBlue $j
      lappend listColormapFilter $j
      lappend listColormapTransmit $j
      lappend listColormapLevel $j
   }
   lset listColormapRed    0 $varRed
   lset listColormapGreen  0 $varGreen
   lset listColormapBlue   0 $varBlue
   updateColormapCanvas
}

proc disableColormap {} {
   global varColormapEnable
   set varColormapEnable 0
}

proc enableColormap {} {
   global stateColormap varColormapEnable
   if { $stateColormap == 0 } {
      disableColormap
   } else {
      set varColormapEnable 1
   }
}

# Scale level command
proc updateColormapLevel { varColormapLevel } {
   global listColormapLevel varCurrentColor

   # zero-based Color Entry
   set zbCE [ expr $varCurrentColor - 1 ]
   lset listColormapLevel $zbCE $varColormapLevel
#   updateColormapCanvas
}

# Scale color-number command
proc updateColormapN { varColormapN } {
   global nColors varCurrentColor varLabelColormapN varEntryColormapN

   set tempCE [ expr int ($nColors * $varColormapN) + 1 ]
   if { $tempCE > $nColors } {
      set tempCE $nColors
   }
   if { $tempCE != $varCurrentColor } {
      set varCurrentColor $tempCE
   }
   if { $varCurrentColor > $nColors } {
     set varCurrentColor $nColors
   }
   updateColors
   updateEntryColormapN $varCurrentColor
   updateRGBCanvas
}

proc updateLabelColormapN { nColors } {
   global varLabelColormapN
   if { $nColors != 1 } {
      set varLabelColormapN "of $nColors colors"
   } else { \
      set varLabelColormapN "of $nColors color "
   }
#      puts "updateLabelColormapN.nColors = $nColors"
}

proc updateEntryColormapN { varCurrentColor } {
   global varEntryColormapN
   set varEntryColormapN $varCurrentColor
}

proc updateColorDel { } {
   global varColorDel stateColormap 
   global varCurrentColor nColors nColorsAdded 
   global varColormapN varLabelColormapN

   set varColorDel 0
#   puts "updateColorDel.stateColormap = $stateColormap\n"
#   puts "updateColorDel.nColors       = $nColors\n"
   if { $stateColormap == 0 } {
   } else { \
      if { $nColorsAdded == 0 } {
      } else { \
         set nColors [ expr $nColors - 1 ]
         set nColorsAdded [ expr $nColorsAdded - 1 ]
         updateLabelColormapN $nColors
         if { $varCurrentColor > $nColors } {
            set varCurrentColor $nColors
#            updateColormapN $varColormapN
         }
      }
   }
   updateColormapN $varColormapN
   updateColormapCanvas
}
 
proc updateColorNew { } {
   global varColorNew stateColormap 
   global varCurrentColor nColors nColorsAdded 
   global varColormapN varLabelColormapN

   set varColorNew 0
#   puts "updateColorNew.stateColormap = $stateColormap\n"
#   puts "updateColorNew.nColors       = $nColors\n"
   if { $stateColormap == 0 } {
   } else { \
      if { $nColors < 256 } {
         set nColors [ expr $nColors + 1 ]
         set nColorsAdded [ expr $nColorsAdded + 1 ]
         updateLabelColormapN $nColors
     }
   }
   updateColormapN $varColormapN
   updateColormapCanvas
}

# Update all colors from color number change.
proc updateColors { } {
   global varRed varGreen varBlue varFilter varTransmit
   global listColormapRed listColormapGreen listColormapBlue 
   global listColormapFilter listColormapTransmit
   global listColormapLevel
   global varCurrentColor varColormapLevel

   if { $varCurrentColor != 0 } {
      set zbCE [ expr $varCurrentColor - 1 ]
      set varRed        [ lindex $listColormapRed $zbCE ]
      set varGreen      [ lindex $listColormapGreen $zbCE ]
      set varBlue       [ lindex $listColormapBlue $zbCE ]
      set varFilter     [ lindex $listColormapFilter $zbCE ]
      set varTransmit   [ lindex $listColormapTransmit $zbCE ]
      set varColormapLevel [ lindex $listColormapLevel $zbCE ]
   }
}

proc updateAmbient { varAmbientRGB } {
   global varAmbientLock varAmbientRed varAmbientGreen varAmbientBlue

   if { $varAmbientLock == 1 } {
      set varAmbientRed   $varAmbientRGB
      set varAmbientGreen $varAmbientRGB
      set varAmbientBlue  $varAmbientRGB
   }
}

proc updateRotation { varAxisRot } {
   global varRotationLock varRotationX varRotationY varRotationZ

   if { $varRotationLock == 1 } {
      set varRotationX $varAxisRot
      set varRotationY $varAxisRot
      set varRotationZ $varAxisRot
   }
}

proc updateScale { varAxisScale } {
   global varScaleLock varScaleX varScaleY varScaleZ

   if { $varScaleLock == 1 } {
      set varScaleX $varAxisScale
      set varScaleY $varAxisScale
      set varScaleZ $varAxisScale
   }
}

proc updateTurbOLO { varAxisTurb } {
   global varWarpEnable 
   global varOctaves varLambda varOmega

   if { $varWarpEnable != 1 } {
         set varOctaves 6
         set varLambda  2.00
         set varOmega   0.50
   }
}

proc updateNoiseGenerator { varNoiseGenerator } {
}

proc updateTurbulence { varAxisTurb } {
   global varWarpEnable varTurbulenceLock 
   global varTurbulenceX varTurbulenceY varTurbulenceZ
   global varOctaves varLambda varOmega

   if { $varWarpEnable == 1 } {
      if { $varTurbulenceLock == 1 } {
         set varTurbulenceX $varAxisTurb
         set varTurbulenceY $varAxisTurb
         set varTurbulenceZ $varAxisTurb
      }
   } else { \
         set varTurbulenceY 0.00
         set varTurbulenceZ 0.00
   }
}

proc updateBlackHoleTurbulence { varAxisBlackHoleTurbulence } {
   global varBlackHoleTurbulenceLock
   global varBlackHoleTurbulenceX varBlackHoleTurbulenceY varBlackHoleTurbulenceZ

   if { $varBlackHoleTurbulenceLock == 1 } {
      set varBlackHoleTurbulenceX $varAxisBlackHoleTurbulence
      set varBlackHoleTurbulenceY $varAxisBlackHoleTurbulence
      set varBlackHoleTurbulenceZ $varAxisBlackHoleTurbulence
   }
}

proc updateBlackHoleRepeat { varAxisBlackHoleRepeat } {
   global varBlackHoleRepeatLock
   global varBlackHoleRepeatX varBlackHoleRepeatY varBlackHoleRepeatZ

   if { $varBlackHoleRepeatLock == 1 } {
      set varBlackHoleRepeatX $varAxisBlackHoleRepeat
      set varBlackHoleRepeatY $varAxisBlackHoleRepeat
      set varBlackHoleRepeatZ $varAxisBlackHoleRepeat
   }
}

proc disableGradient {} {
   global varGradientEnable varGradientX varGradientY varGradientZ
   set varGradientEnable 0
   set varGradientX 0
   set varGradientY 0
   set varGradientZ 0
}

proc enableGradient { } {
   global stateGradient varGradientEnable varGradientX varGradientY varGradientZ
   if { $stateGradient == 0 } {
      disableGradient
   } else {
      set varGradientEnable 1
      enableGradientX
#      .frameMain.cbGradientX configure -text "x"
#      .frameMain.cbGradientY configure -text "y"
#      .frameMain.cbGradientZ configure -text "z"
#      labelFractalWarp
   }
}
proc enableGradientX {} {
   global stateGradient varGradientX varGradientY varGradientZ
   if { $stateGradient == 0 } {
      disableGradient
   } else {
      set varGradientX 1
      set varGradientY 0
      set varGradientZ 0
   }
}

proc enableGradientY {} {
   global stateGradient varGradientX varGradientY varGradientZ
   if { $stateGradient == 0 } {
      disableGradient
   } else {
      set varGradientX 0
      set varGradientY 1
      set varGradientZ 0
   }
}

proc enableGradientZ {} {
   global stateGradient varGradientX varGradientY varGradientZ
   if { $stateGradient == 0 } {
      disableGradient
   } else {
      set varGradientX 0
      set varGradientY 0
      set varGradientZ 1
   }
}

proc enableOrientationX {} {
   global varCSTOrientationX varCSTOrientationY varCSTOrientationZ
   set varCSTOrientationX 1
   set varCSTOrientationY 0
   set varCSTOrientationZ 0
}

proc enableOrientationY {} {
   global varCSTOrientationX varCSTOrientationY varCSTOrientationZ
   set varCSTOrientationX 0
   set varCSTOrientationY 1
   set varCSTOrientationZ 0
}

proc enableOrientationZ {} {
   global varCSTOrientationX varCSTOrientationY varCSTOrientationZ
   set varCSTOrientationX 0
   set varCSTOrientationY 0
   set varCSTOrientationZ 1
}

proc enableDirectionX {} {
   global varRepeatDirectionX varRepeatDirectionY varRepeatDirectionZ
   set varRepeatDirectionX 1
   set varRepeatDirectionY 0
   set varRepeatDirectionZ 0
}

proc enableDirectionY {} {
   global varRepeatDirectionX varRepeatDirectionY varRepeatDirectionZ
   set varRepeatDirectionX 0
   set varRepeatDirectionY 1
   set varRepeatDirectionZ 0
}

proc enableDirectionZ {} {
   global varRepeatDirectionX varRepeatDirectionY varRepeatDirectionZ
   set varRepeatDirectionX 0
   set varRepeatDirectionY 0
   set varRepeatDirectionZ 1
}

proc execExit {} {
   exit
}

# GIF and PPM are supported but not PNG or JPG.
# See error logs, POV-Ray renderings produce GIF and PPM that are not recognized properly,
# "post-processing" by invoking ImageMagick convert to open & save produces working GIF and PPM.
# 
proc execRender {} {
   global filePOV varLabelTextRendering
   global varPattern varEnableFullScreen
   global varHiQ

   set xlocImg   120
   set ylocImg   100
   set widthImg  300
   set heightImg 225
   set buttonBack #33CC99
   set buttonFore #FFFFFF
   set buttonGrey #D9D9D9

# Attempt to show AltR has virtually pressed the Render button. 
#   .frameMain.buttonRender configure -relief sunken
#   .frameMain.buttonRender configure -state active
#
#	.frameMain.labelRendering configure -background $buttonBack
#	.frameMain.labelRendering configure -foreground $buttonFore
   set varLabelTextRendering "Rendering"
   set ob 91
   set cb 93
#   updateVars

#   writePOV $filePOV
#      catch { exec povray fullscreen.ini }
   writePOVtmp "tkxture.inc"
   if { $varEnableFullScreen == 1 } {
      if { $varHiQ == 1 } {
         catch { exec povray tFShigh.ini }
      } else {
         catch { exec povray tFSlow.ini }
      } 
   } else { \
      if { $varHiQ == 1 } {
         catch { exec povray tGUIhigh.ini }
      } else {
         catch { exec povray tGUIlow.ini  }
      } 
   }
   # wait here or the convert will be starved!!!
   catch { exec convert tkxture.png tkxture.ppm }
   # wait here or the set will be starved!!!
   set imageRender [image create photo -format ppm -width $widthImg -height $heightImg -file tkxture.ppm]

   .frameMain.canvasRendering create image $xlocImg $ylocImg -image $imageRender

# Attempt to show AltR initiated rendering has finished.
#   .frameMain.buttonRender configure -relief raised
#   .frameMain.buttonRender configure -state normal
#
#	.frameMain.labelRendering configure -background $buttonGrey
#	.frameMain.labelRendering configure -foreground $buttonGrey
      set varLabelTextRendering "         "
}

proc updateRed { varRed } {
   global listColormapRed  varCurrentColor

   if { $varCurrentColor != 0 } {
      set zbCE [ expr $varCurrentColor - 1 ]
      lset listColormapRed $zbCE $varRed
      updateRGBCanvas
   }
}

proc updateGreen { varGreen } {
   global listColormapGreen  varCurrentColor

   if { $varCurrentColor != 0 } {
      set zbCE [ expr $varCurrentColor - 1 ]
      lset listColormapGreen $zbCE $varGreen
      updateRGBCanvas
   }
}

proc updateBlue { varBlue } {
   global listColormapBlue  varCurrentColor

   if { $varCurrentColor != 0 } {
      set zbCE [ expr $varCurrentColor - 1 ]
      lset listColormapBlue $zbCE $varBlue
      updateRGBCanvas
   }
}

proc updateFilter { varFilter } {
   global listColormapFilter  varCurrentColor

   if { $varCurrentColor != 0 } {
      set zbCE [ expr $varCurrentColor - 1 ]
      lset listColormapFilter $zbCE $varFilter 
   }
}

proc updateTransmit { varTransmit } {
   global listColormapTransmit  varCurrentColor

   if { $varCurrentColor != 0 } {
      set zbCE [ expr $varCurrentColor - 1 ]
      lset listColormapTransmit $zbCE $varTransmit 
   }
}

# Pattern modifier updates
proc updateBrick { varAxisBrick } {
   global varBrickLock varBrickX varBrickY varBrickZ

   if { $varBrickLock == 1 } {
      set varBrickX $varAxisBrick
      set varBrickY $varAxisBrick
      set varBrickZ $varAxisBrick
   }
}

# Set patern type and associated variables.
# First row is Pattern menu, followed by ten available control rows in Pattern window.
set lastPatternRow 11

proc erasePatternControls { } {

   set listControls { abc }
   set listControls [ grid slaves .frameMain.framePattern ]
   set nControls [ llength $listControls ]

   # Adjust, save the last 4; label4Pattern1, mbPattern, entryPattern, label10Pattern1.
   set nControls [ expr $nControls - 4 ]

#   puts "listControls = $listControls\n"
   while { $nControls > 0 } {
      set slaveCtrl [ lindex $listControls [expr $nControls - 1 ] ]
#      puts "slaveCtrl = $slaveCtrl\n"
	   grid remove $slaveCtrl

      set nControls [ expr $nControls - 1 ]
   }
}

proc addNoiseGen { } {
   set topRow 2

	grid configure .frameMain.labelNoiseGenerator   -in .frameMain.framePattern	-row $topRow -column 1 -columnspan 2 -sticky e
	grid configure .frameMain.scaleNoiseGenerator   -in .frameMain.framePattern	-row $topRow -column 3 -columnspan 2 -sticky ew
	grid configure .frameMain.entryNoiseGenerator   -in .frameMain.framePattern	-row $topRow -column 5 

}

# Fill Pattern control rows with blanks.
proc fillPatternBlanks { thisRow lastRow } {

   while { $thisRow <= $lastRow } {
	   grid configure .frameMain.label14Pattern$thisRow -in .frameMain.framePattern	-row $thisRow -column 1 -columnspan 2 -sticky e
	   grid configure .frameMain.label15Pattern$thisRow -in .frameMain.framePattern	-row $thisRow -column 3 -columnspan 2 -sticky ew
	   grid configure .frameMain.label6Pattern$thisRow  -in .frameMain.framePattern	-row $thisRow -column 5

      set thisRow [ expr $thisRow + 1 ]
   }
}

# Set pattern-specific flags and GUI.

# Non colormap patterns brick, checker, color, hexagon, image map.
proc setBrick {} {
   global activePattern lastPatternRow stateNoiseGenerator

   set activePattern "brick"
   set stateNoiseGenerator 0

   erasePatternControls

   set currentRow 2 
   fillPatternBlanks $currentRow $currentRow

   set currentRow [ expr $currentRow + 1 ]
	grid .frameMain.labelBrickX -in .frameMain.framePattern	-row $currentRow -column 1 -columnspan 2 -sticky e
	grid .frameMain.scaleBrickX -in .frameMain.framePattern	-row $currentRow -column 3 -columnspan 2 -sticky ew
	grid .frameMain.entryBrickX -in .frameMain.framePattern	-row $currentRow -column 5

   set currentRow [ expr $currentRow + 1 ]
	grid .frameMain.cbLockBrickY -in .frameMain.framePattern -row $currentRow -column 1 -columnspan 2 -sticky e
	grid .frameMain.scaleBrickY -in .frameMain.framePattern	-row $currentRow -column 3  -columnspan 2 -sticky ew
	grid .frameMain.entryBrickY -in .frameMain.framePattern	-row $currentRow -column 5 

   set currentRow [ expr $currentRow + 1 ]
	grid .frameMain.labelBrickZ -in .frameMain.framePattern	-row $currentRow -column 1 -columnspan 2 -sticky e
	grid .frameMain.scaleBrickZ -in .frameMain.framePattern	-row $currentRow -column 3 -columnspan 2 -sticky ew
	grid .frameMain.entryBrickZ -in .frameMain.framePattern	-row $currentRow -column 5 

   set currentRow [ expr $currentRow + 1 ]
	grid .frameMain.labelMortar -in .frameMain.framePattern	-row $currentRow -column 1 -columnspan 2 -sticky e
	grid .frameMain.scaleMortar -in .frameMain.framePattern	-row $currentRow -column 3 -columnspan 2 -sticky ew
	grid .frameMain.entryMortar -in .frameMain.framePattern	-row $currentRow -column 5 

   set currentRow [ expr $currentRow + 1 ]
   fillPatternBlanks $currentRow $lastPatternRow

   setPattern
}

proc setChecker {} {
   global activePattern lastPatternRow stateNoiseGenerator

   set activePattern "checker"
   set stateNoiseGenerator 0

   erasePatternControls 

   set currentRow 2
   fillPatternBlanks $currentRow $lastPatternRow

   setPattern
}

proc setColor {} {
   global activePattern lastPatternRow stateNoiseGenerator

   set activePattern "color"
   set stateNoiseGenerator 0

   erasePatternControls
 
   set currentRow 2
   fillPatternBlanks $currentRow $lastPatternRow

   setPattern
}

proc setHexagon {} {
   global activePattern lastPatternRow stateNoiseGenerator

   set activePattern "hexagon"
   set stateNoiseGenerator 0

   erasePatternControls 

   set currentRow 2
   fillPatternBlanks $currentRow $lastPatternRow

   setPattern
}

proc setImageMap {} {
   global activePattern lastPatternRow stateNoiseGenerator

   set activePattern "image map"
   set stateNoiseGenerator 0

   erasePatternControls

   set currentRow 2
	grid .frameMain.buttonImage -in .frameMain.framePattern	-row $currentRow -column 1 -columnspan 2 -sticky e
	grid .frameMain.entryImage  -in .frameMain.framePattern	-row $currentRow -column 3	-columnspan 2 -sticky ew

   set currentRow [ expr $currentRow + 1 ]
	grid .frameMain.cbImageMapOnce -in .frameMain.framePattern	-row $currentRow -column 3 -columnspan 2 -sticky w

   set currentRow [ expr $currentRow + 1 ]
	grid .frameMain.labelImageMapType -in .frameMain.framePattern	-row $currentRow -column 1 -columnspan 2 -sticky e
	grid .frameMain.cbImageMapPlanar  -in .frameMain.framePattern	-row $currentRow -column 3	-columnspan 1 -sticky w
	grid .frameMain.cbImageMapSphere  -in .frameMain.framePattern	-row $currentRow -column 4	-columnspan 2 -sticky w

   set currentRow [ expr $currentRow + 1 ]
	grid .frameMain.cbImageMapCylinder  -in .frameMain.framePattern	-row $currentRow -column 3	-columnspan 1 -sticky w
	grid .frameMain.cbImageMapTorus  -in .frameMain.framePattern	-row $currentRow -column 4	-columnspan 2 -sticky w

   set currentRow [ expr $currentRow + 1 ]
	grid .frameMain.labelInterpolate -in .frameMain.framePattern	-row $currentRow -column 1 -columnspan 2 -sticky e
	grid .frameMain.cbInterpNorm  -in .frameMain.framePattern	-row $currentRow -column 3	-columnspan 1 -sticky w
	grid .frameMain.cbInterpBilinear  -in .frameMain.framePattern	-row $currentRow -column 4	-columnspan 2 -sticky w

   set currentRow [ expr $currentRow + 1 ]
   fillPatternBlanks $currentRow $lastPatternRow

   setPattern
}

# Colormap patterns
proc setAgate {} {
   global activePattern lastPatternRow stateNoiseGenerator

   set activePattern "agate"
   set stateNoiseGenerator 1

   erasePatternControls 

   addNoiseGen

   set currentRow 3
	grid configure .frameMain.labelAgateTurbulence  -in .frameMain.framePattern	-row $currentRow -column 1 -columnspan 2 -sticky e
	grid configure .frameMain.scaleAgateTurbulence  -in .frameMain.framePattern	-row $currentRow -column 3 -columnspan 2 -sticky ew
	grid configure .frameMain.entryAgateTurbulence  -in .frameMain.framePattern	-row $currentRow -column 5

   set currentRow [ expr $currentRow + 1 ] 
   fillPatternBlanks $currentRow $lastPatternRow

   setPattern
}

proc setBozo {} {
   global activePattern lastPatternRow stateNoiseGenerator

   set activePattern "bozo"
   set stateNoiseGenerator 1

   erasePatternControls

   addNoiseGen

   set currentRow 3
   fillPatternBlanks $currentRow $lastPatternRow

   setPattern
}


proc setCrackle {} {
   global activePattern lastPatternRow stateNoiseGenerator

   set activePattern "crackle"
   set stateNoiseGenerator 0

   erasePatternControls 

   set currentRow 2
   fillPatternBlanks $currentRow $currentRow

   set currentRow 3
	grid configure .frameMain.labelCrackleForm1  -in .frameMain.framePattern	-row $currentRow -column 1 -columnspan 2 -sticky e
	grid configure .frameMain.scaleCrackleForm1  -in .frameMain.framePattern	-row $currentRow -column 3 -columnspan 2 -sticky ew
	grid configure .frameMain.entryCrackleForm1  -in .frameMain.framePattern	-row $currentRow -column 5

   set currentRow [ expr $currentRow + 1 ] 
	grid configure .frameMain.labelCrackleForm2  -in .frameMain.framePattern	-row $currentRow -column 1 -columnspan 2 -sticky e
	grid configure .frameMain.scaleCrackleForm2  -in .frameMain.framePattern	-row $currentRow -column 3 -columnspan 2 -sticky ew
	grid configure .frameMain.entryCrackleForm2  -in .frameMain.framePattern	-row $currentRow -column 5

   set currentRow [ expr $currentRow + 1 ] 
	grid configure .frameMain.labelCrackleForm3  -in .frameMain.framePattern	-row $currentRow -column 1 -columnspan 2 -sticky e
	grid configure .frameMain.scaleCrackleForm3  -in .frameMain.framePattern	-row $currentRow -column 3 -columnspan 2 -sticky ew
	grid configure .frameMain.entryCrackleForm3  -in .frameMain.framePattern	-row $currentRow -column 5

   set currentRow [ expr $currentRow + 1 ] 
	grid configure .frameMain.cbCrackleSolid  -in .frameMain.framePattern	-row $currentRow -column 1 -columnspan 2 -sticky e

   set currentRow [ expr $currentRow + 1 ] 
	grid configure .frameMain.labelCrackleMetric -in .frameMain.framePattern	-row $currentRow -column 1 -columnspan 2 -sticky e
	grid configure .frameMain.scaleCrackleMetric -in .frameMain.framePattern	-row $currentRow -column 3 -columnspan 2 -sticky ew
	grid configure .frameMain.entryCrackleMetric -in .frameMain.framePattern	-row $currentRow -column 5

   set currentRow [ expr $currentRow + 1 ] 
	grid configure .frameMain.labelCrackleOffset -in .frameMain.framePattern	-row $currentRow -column 1 -columnspan 2 -sticky e
	grid configure .frameMain.scaleCrackleOffset -in .frameMain.framePattern	-row $currentRow -column 3 -columnspan 2 -sticky ew
	grid configure .frameMain.entryCrackleOffset -in .frameMain.framePattern	-row $currentRow -column 5

   set currentRow [ expr $currentRow + 1 ] 
   fillPatternBlanks $currentRow $lastPatternRow

   setPattern
}

proc setGradient {} {
   global activePattern lastPatternRow stateNoiseGenerator

   set activePattern "gradient"
   set stateNoiseGenerator 0

   erasePatternControls

   set currentRow 2
   fillPatternBlanks $currentRow $currentRow

   set currentRow [ expr $currentRow + 1 ]
	grid .frameMain.labelGradient   -in .frameMain.framePattern	-row $currentRow -column 1 -columnspan 2 -sticky e
	grid .frameMain.cbGradientX     -in .frameMain.framePattern	-row $currentRow -column 3	-sticky ew
	grid .frameMain.cbGradientY     -in .frameMain.framePattern	-row $currentRow -column 4	-sticky ew
	grid .frameMain.cbGradientZ     -in .frameMain.framePattern	-row $currentRow -column 5	-sticky ew

   set currentRow [ expr $currentRow + 1 ]
   fillPatternBlanks $currentRow $lastPatternRow

   setPattern
}

proc setGranite {} {
   global activePattern lastPatternRow stateNoiseGenerator

   set activePattern "granite"
   set stateNoiseGenerator 1

   erasePatternControls
 
   addNoiseGen

   set currentRow 3
   fillPatternBlanks $currentRow $lastPatternRow

   setPattern
}

proc setJulia {} {
   global activePattern lastPatternRow stateNoiseGenerator

   set activePattern "julia"
   set stateNoiseGenerator 0

   erasePatternControls

   set currentRow 2
   fillPatternBlanks $currentRow $currentRow 

   set currentRow [ expr $currentRow + 1 ]
	grid .frameMain.labelFractalExponent -in .frameMain.framePattern	-row $currentRow -column 1 -columnspan 2 -sticky e
	grid .frameMain.scaleFractalExponent -in .frameMain.framePattern	-row $currentRow -column 3 -columnspan 2 -sticky ew
	grid .frameMain.entryFractalExponent -in .frameMain.framePattern	-row $currentRow -column 5

   set currentRow [ expr $currentRow + 1 ]
	grid .frameMain.labelFractalterations -in .frameMain.framePattern	-row $currentRow -column 1  -columnspan 2 -sticky e
	grid .frameMain.scaleFractalterations -in .frameMain.framePattern	-row $currentRow -column 3  -columnspan 2 -sticky ew
	grid .frameMain.entryFractalterations -in .frameMain.framePattern	-row $currentRow -column 5 

   set currentRow [ expr $currentRow + 1 ]
	grid .frameMain.labelFractalMagnetType -in .frameMain.framePattern	-row $currentRow -column 1 -columnspan 2 -sticky e
	grid .frameMain.scaleFractalMagnetType -in .frameMain.framePattern	-row $currentRow -column 3  -columnspan 2 -sticky ew
	grid .frameMain.entryFractalMagnetType -in .frameMain.framePattern	-row $currentRow -column 5 

   set currentRow [ expr $currentRow + 1 ]
	grid .frameMain.labelFractalExteriorType -in .frameMain.framePattern	-row $currentRow -column 1 -columnspan 2 -sticky e
	grid .frameMain.scaleFractalExteriorType -in .frameMain.framePattern	-row $currentRow -column 3  -columnspan 2 -sticky ew
	grid .frameMain.entryFractalExteriorType -in .frameMain.framePattern	-row $currentRow -column 5 

   set currentRow [ expr $currentRow + 1 ]
	grid .frameMain.labelFractalExteriorFactor -in .frameMain.framePattern	-row $currentRow -column 1 -columnspan 2 -sticky e
	grid .frameMain.scaleFractalExteriorFactor -in .frameMain.framePattern	-row $currentRow -column 3 -columnspan 2 -sticky ew
	grid .frameMain.entryFractalExteriorFactor -in .frameMain.framePattern	-row $currentRow -column 5 

   set currentRow [ expr $currentRow + 1 ]
	grid .frameMain.labelFractalInteriorType -in .frameMain.framePattern	-row $currentRow -column 1  -columnspan 2 -sticky e
	grid .frameMain.scaleFractalInteriorType -in .frameMain.framePattern	-row $currentRow -column 3  -columnspan 2 -sticky ew
	grid .frameMain.entryFractalInteriorType -in .frameMain.framePattern	-row $currentRow -column 5 

   set currentRow [ expr $currentRow + 1 ]
	grid .frameMain.labelFractalInteriorFactor -in .frameMain.framePattern	-row $currentRow -column 1 -columnspan 2 -sticky e
	grid .frameMain.scaleFractalInteriorFactor -in .frameMain.framePattern	-row $currentRow -column 3 -columnspan 2 -sticky ew
	grid .frameMain.entryFractalInteriorFactor -in .frameMain.framePattern	-row $currentRow -column 5 

   set currentRow [ expr $currentRow + 1 ]
	grid .frameMain.labelFractalComplexA -in .frameMain.framePattern	-row $currentRow -column 1 -columnspan 2 -sticky e
	grid .frameMain.scaleFractalComplexA -in .frameMain.framePattern	-row $currentRow -column 3  -columnspan 2 -sticky ew
	grid .frameMain.entryFractalComplexA -in .frameMain.framePattern	-row $currentRow -column 5 

   set currentRow [ expr $currentRow + 1 ]
	grid .frameMain.labelFractalComplexB -in .frameMain.framePattern	-row $currentRow -column 1 -columnspan 2 -sticky e
	grid .frameMain.scaleFractalComplexB -in .frameMain.framePattern	-row $currentRow -column 3 -columnspan 2 -sticky ew
	grid .frameMain.entryFractalComplexB -in .frameMain.framePattern	-row $currentRow -column 5 

   setPattern
}

proc setLeopard {} {
   global activePattern lastPatternRow stateNoiseGenerator

   set activePattern "leopard"
   set stateNoiseGenerator 1

   erasePatternControls

   addNoiseGen

   set currentRow 3
   fillPatternBlanks $currentRow $lastPatternRow

   setPattern
}

proc setMandel {} {
   global activePattern lastPatternRow stateNoiseGenerator

   set activePattern "mandelbrot"
   set stateNoiseGenerator 0

   erasePatternControls

   set currentRow 2
   fillPatternBlanks $currentRow $currentRow 

   set currentRow [ expr $currentRow + 1 ]
	grid .frameMain.labelFractalExponent -in .frameMain.framePattern	-row $currentRow -column 1 -columnspan 2 -sticky e
	grid .frameMain.scaleFractalExponent -in .frameMain.framePattern	-row $currentRow -column 3 -columnspan 2 -sticky ew
	grid .frameMain.entryFractalExponent -in .frameMain.framePattern	-row $currentRow -column 5

   set currentRow [ expr $currentRow + 1 ]
	grid .frameMain.labelFractalterations -in .frameMain.framePattern	-row $currentRow -column 1  -columnspan 2 -sticky e
	grid .frameMain.scaleFractalterations -in .frameMain.framePattern	-row $currentRow -column 3  -columnspan 2 -sticky ew
	grid .frameMain.entryFractalterations -in .frameMain.framePattern	-row $currentRow -column 5 

   set currentRow [ expr $currentRow + 1 ]
	grid .frameMain.labelFractalMagnetType -in .frameMain.framePattern	-row $currentRow -column 1 -columnspan 2 -sticky e
	grid .frameMain.scaleFractalMagnetType -in .frameMain.framePattern	-row $currentRow -column 3  -columnspan 2 -sticky ew
	grid .frameMain.entryFractalMagnetType -in .frameMain.framePattern	-row $currentRow -column 5 

   set currentRow [ expr $currentRow + 1 ]
	grid .frameMain.labelFractalExteriorType -in .frameMain.framePattern	-row $currentRow -column 1 -columnspan 2 -sticky e
	grid .frameMain.scaleFractalExteriorType -in .frameMain.framePattern	-row $currentRow -column 3  -columnspan 2 -sticky ew
	grid .frameMain.entryFractalExteriorType -in .frameMain.framePattern	-row $currentRow -column 5 

   set currentRow [ expr $currentRow + 1 ]
	grid .frameMain.labelFractalExteriorFactor -in .frameMain.framePattern	-row $currentRow -column 1 -columnspan 2 -sticky e
	grid .frameMain.scaleFractalExteriorFactor -in .frameMain.framePattern	-row $currentRow -column 3 -columnspan 2 -sticky ew
	grid .frameMain.entryFractalExteriorFactor -in .frameMain.framePattern	-row $currentRow -column 5 

   set currentRow [ expr $currentRow + 1 ]
	grid .frameMain.labelFractalInteriorType -in .frameMain.framePattern	-row $currentRow -column 1  -columnspan 2 -sticky e
	grid .frameMain.scaleFractalInteriorType -in .frameMain.framePattern	-row $currentRow -column 3  -columnspan 2 -sticky ew
	grid .frameMain.entryFractalInteriorType -in .frameMain.framePattern	-row $currentRow -column 5 

   set currentRow [ expr $currentRow + 1 ]
	grid .frameMain.labelFractalInteriorFactor -in .frameMain.framePattern	-row $currentRow -column 1 -columnspan 2 -sticky e
	grid .frameMain.scaleFractalInteriorFactor -in .frameMain.framePattern	-row $currentRow -column 3 -columnspan 2 -sticky ew
	grid .frameMain.entryFractalInteriorFactor -in .frameMain.framePattern	-row $currentRow -column 5 

   set currentRow [ expr $currentRow + 1 ]
   fillPatternBlanks $currentRow $lastPatternRow

   setPattern
}

proc setMarble {} {
   global activePattern lastPatternRow stateNoiseGenerator

   set activePattern "marble"
   set stateNoiseGenerator 0

   erasePatternControls
 
   set currentRow 2
   fillPatternBlanks $currentRow $lastPatternRow

   setPattern
}

proc setObject {} {
   global activePattern lastPatternRow stateNoiseGenerator

   set activePattern "object"
   set stateNoiseGenerator 0

   erasePatternControls
 
   set currentRow 2
   fillPatternBlanks $currentRow $lastPatternRow

   setPattern
}

proc setRadial {} {
   global activePattern lastPatternRow stateNoiseGenerator

   set activePattern "radial"
   set stateNoiseGenerator 0

   erasePatternControls 

   set currentRow 2
   fillPatternBlanks $currentRow $lastPatternRow

   setPattern
}

proc setSpotted {} {
   global activePattern lastPatternRow

   set activePattern "spotted"
   set stateNoiseGenerator 1

   erasePatternControls

   addNoiseGen

   set currentRow 3
   fillPatternBlanks $currentRow $lastPatternRow

   setPattern
}

proc setWaves {} {
   global activePattern lastPatternRow stateNoiseGenerator

   set activePattern "waves"
   set stateNoiseGenerator 1

   erasePatternControls

   addNoiseGen

   set currentRow 3
   fillPatternBlanks $currentRow $lastPatternRow

   setPattern
}

proc setWood {} {
   global activePattern lastPatternRow stateNoiseGenerator

   set activePattern "wood"
   set stateNoiseGenerator 0

   erasePatternControls 

   set currentRow 2
   fillPatternBlanks $currentRow $lastPatternRow

   setPattern
}

proc setWrinkles {} {
   global activePattern lastPatternRow stateNoiseGenerator

   set activePattern "wrinkles"
   set stateNoiseGenerator 1

   erasePatternControls

   addNoiseGen

   set currentRow 3
   fillPatternBlanks $currentRow $lastPatternRow

   setPattern
}

# Set warp type and associated variables.
set lastWarpRow 7

proc eraseWarpControls { } {

   set listControls { abc }
   set listControls [ grid slaves .frameMain.frameWarp ]
   set nControls [ llength $listControls ]

   # Adjust, save the last n;
   # cdWarp, mbWarp, entryWarp,
   # cbWarpBeforeTurb labelWarpPriority cbWarpAfterTurb labelTurb
   set nControls [ expr $nControls - 7 ]

#   puts "listControls = $listControls\n"
   while { $nControls > 0 } {
      set slaveCtrl [ lindex $listControls [expr $nControls - 1 ] ]
#      puts "slaveCtrl = $slaveCtrl\n"
	   grid remove $slaveCtrl

      set nControls [ expr $nControls - 1 ]
   }
}

# Fill Warp control rows with blanks.
proc fillWarpBlanks { thisRow lastRow leftColumn } {

   if { $leftColumn == 1 } {

      while { $thisRow <= $lastRow } {
	      grid configure .frameMain.labelL14Warp$thisRow -in .frameMain.frameWarp	-row $thisRow -column $leftColumn -columnspan 2 -sticky e
         set leftColumn [ expr $leftColumn + 2 ]
	      grid configure .frameMain.labelL15Warp$thisRow -in .frameMain.frameWarp	-row $thisRow -column $leftColumn -columnspan 2 -sticky ew
         set leftColumn [ expr $leftColumn + 2 ]
	      grid configure .frameMain.labelL6Warp$thisRow  -in .frameMain.frameWarp	-row $thisRow -column $leftColumn

         set leftColumn [ expr $leftColumn - 4 ]
         set thisRow [ expr $thisRow + 1 ]
      }
   } else {
      while { $thisRow <= $lastRow } {
	      grid configure .frameMain.labelR14Warp$thisRow -in .frameMain.frameWarp	-row $thisRow -column $leftColumn -columnspan 2 -sticky e
         set leftColumn [ expr $leftColumn + 2 ]
	      grid configure .frameMain.labelR15Warp$thisRow -in .frameMain.frameWarp	-row $thisRow -column $leftColumn -columnspan 2 -sticky ew
         set leftColumn [ expr $leftColumn + 2 ]
	      grid configure .frameMain.labelR6Warp$thisRow  -in .frameMain.frameWarp	-row $thisRow -column $leftColumn

         set leftColumn [ expr $leftColumn - 4 ]
         set thisRow [ expr $thisRow + 1 ]
      }
   }
}

# Set translate priority plus before and after displays
proc setTranslateBeforeWarp {} {
   global varTranslateBeforeWarp varTranslateAfterWarp
   set varTranslateBeforeWarp 1
   set varTranslateAfterWarp  0

   setTranslatePriority
}

proc setTranslateAfterWarp {} {
   global varTranslateBeforeWarp varTranslateAfterWarp
   set varTranslateBeforeWarp 0
   set varTranslateAfterWarp  1

   setTranslatePriority
}

proc setTranslatePriority { } {
   global varTranslateBeforeWarp varTranslateAfterWarp
   global varTranslateWarpPriority varLabelTextWarpTurb
   if { $varTranslateBeforeWarp == "1" } {
      set varTranslateWarpPriority "before"
      set varLabelTextWarpTurb "warp turbulence"
   } else { \
      set varTranslateWarpPriority "after "
      set varLabelTextWarpTurb "warp turbulence"
   }
}

# Set warp priority plus before and after displays
proc setWarpBeforeTurb {} {
   global varWarpBeforeTurb varWarpAfterTurb
   set varWarpBeforeTurb 1
   set varWarpAfterTurb  0

   setWarpPriority
}

proc setWarpAfterTurb {} {
   global varWarpBeforeTurb varWarpAfterTurb
   set varWarpBeforeTurb 0
   set varWarpAfterTurb  1

   setWarpPriority
}

proc setWarpPriority { } {
   global activeWarp varWarpBeforeTurb
   global varLabelTextTurb varWarpTurbPriority

   if { $activeWarp == "turbulence" } {
         set varWarpTurbPriority "only  "
         set varLabelTextTurb "          "
   } else { \
      if { $varWarpBeforeTurb == "1" } {
         set varWarpTurbPriority "before"
         set varLabelTextTurb "turbulence"
      } else { \
         set varWarpBeforeTurb 0
         set varWarpTurbPriority "after "
         set varLabelTextTurb "turbulence"
      }
   }
}

# Set warp type.
proc setWarpBlackhole {} {
   global activeWarp lastWarpRow

   set activeWarp "blackhole"

   setWarpPriority
   eraseWarpControls

   # Left
   set currentRow 2
	grid .frameMain.labelBlackHoleLocationX -in .frameMain.frameWarp	-row $currentRow -column 1 -columnspan 2 -sticky e
	grid .frameMain.scaleBlackHoleLocationX -in .frameMain.frameWarp	-row $currentRow -column 3 -columnspan 2 -sticky ew
	grid .frameMain.entryBlackHoleLocationX -in .frameMain.frameWarp	-row $currentRow -column 5 

   set currentRow [ expr $currentRow + 1 ]
	grid .frameMain.labelBlackHoleLocationY -in .frameMain.frameWarp	-row $currentRow -column 1 -columnspan 2 -sticky e
	grid .frameMain.scaleBlackHoleLocationY -in .frameMain.frameWarp	-row $currentRow -column 3 -columnspan 2 -sticky ew
	grid .frameMain.entryBlackHoleLocationY -in .frameMain.frameWarp	-row $currentRow -column 5 

   set currentRow [ expr $currentRow + 1 ]
	grid .frameMain.labelBlackHoleLocationZ -in .frameMain.frameWarp	-row $currentRow -column 1 -columnspan 2 -sticky e
	grid .frameMain.scaleBlackHoleLocationZ -in .frameMain.frameWarp	-row $currentRow -column 3 -columnspan 2 -sticky ew
	grid .frameMain.entryBlackHoleLocationZ -in .frameMain.frameWarp	-row $currentRow -column 5 

   set currentRow [ expr $currentRow + 1 ]
	grid .frameMain.labelBlackHoleTurbulenceX -in .frameMain.frameWarp	-row $currentRow -column 1 -columnspan 2 -sticky e
	grid .frameMain.scaleBlackHoleTurbulenceX -in .frameMain.frameWarp	-row $currentRow -column 3 -columnspan 2 -sticky ew
	grid .frameMain.entryBlackHoleTurbulenceX -in .frameMain.frameWarp	-row $currentRow -column 5 

   set currentRow [ expr $currentRow + 1 ]
	grid .frameMain.cbLockBlackHoleTurbulenceY -in .frameMain.frameWarp	-row $currentRow -column 1 -columnspan 2 -sticky e
	grid .frameMain.scaleBlackHoleTurbulenceY -in .frameMain.frameWarp	-row $currentRow -column 3 -columnspan 2 -sticky ew
	grid .frameMain.entryBlackHoleTurbulenceY -in .frameMain.frameWarp	-row $currentRow -column 5 

   set currentRow [ expr $currentRow + 1 ]
	grid .frameMain.labelBlackHoleTurbulenceZ -in .frameMain.frameWarp	-row $currentRow -column 1 -columnspan 2 -sticky e
	grid .frameMain.scaleBlackHoleTurbulenceZ -in .frameMain.frameWarp	-row $currentRow -column 3 -columnspan 2 -sticky ew
	grid .frameMain.entryBlackHoleTurbulenceZ -in .frameMain.frameWarp	-row $currentRow -column 5 

   # Right
   set currentRow 2
	grid .frameMain.labelBlackHoleRadius -in .frameMain.frameWarp	-row $currentRow -column 6 -columnspan 2 -sticky e
	grid .frameMain.scaleBlackHoleRadius -in .frameMain.frameWarp	-row $currentRow -column 8  -columnspan 2 -sticky ew
	grid .frameMain.entryBlackHoleRadius -in .frameMain.frameWarp	-row $currentRow -column 10 

   set currentRow [ expr $currentRow + 1 ]
	grid .frameMain.cbInvertBlackHoleFalloff -in .frameMain.frameWarp	-row $currentRow -column 6 -columnspan 2 -sticky e
	grid .frameMain.scaleBlackHoleFalloff -in .frameMain.frameWarp	-row $currentRow -column 8  -columnspan 2 -sticky ew
	grid .frameMain.entryBlackHoleFalloff -in .frameMain.frameWarp	-row $currentRow -column 10 

   set currentRow [ expr $currentRow + 1 ]
	grid .frameMain.labelBlackHoleStrength -in .frameMain.frameWarp	-row $currentRow -column 6 -columnspan 2 -sticky e
	grid .frameMain.scaleBlackHoleStrength -in .frameMain.frameWarp	-row $currentRow -column 8  -columnspan 2 -sticky ew
	grid .frameMain.entryBlackHoleStrength -in .frameMain.frameWarp	-row $currentRow -column 10 

   set currentRow [ expr $currentRow + 1 ]
	grid .frameMain.labelBlackHoleRepeatX -in .frameMain.frameWarp	-row $currentRow -column 6 -columnspan 2 -sticky e
	grid .frameMain.scaleBlackHoleRepeatX -in .frameMain.frameWarp	-row $currentRow -column 8  -columnspan 2 -sticky ew
	grid .frameMain.entryBlackHoleRepeatX -in .frameMain.frameWarp	-row $currentRow -column 10 

   set currentRow [ expr $currentRow + 1 ]
	grid .frameMain.cbLockBlackHoleRepeatY -in .frameMain.frameWarp	-row $currentRow -column 6 -columnspan 2 -sticky e
	grid .frameMain.scaleBlackHoleRepeatY -in .frameMain.frameWarp	-row $currentRow -column 8  -columnspan 2 -sticky ew
	grid .frameMain.entryBlackHoleRepeatY -in .frameMain.frameWarp	-row $currentRow -column 10 

   set currentRow [ expr $currentRow + 1 ]
#	grid .frameMain.labelBlackHoleRepeatZ -in .frameMain.frameWarp	-row $currentRow -column 6 -columnspan 2 -sticky e
	grid .frameMain.cbX10BlackHoleRepeatZ -in .frameMain.frameWarp	-row $currentRow -column 6 -columnspan 2 -sticky e
	grid .frameMain.scaleBlackHoleRepeatZ -in .frameMain.frameWarp	-row $currentRow -column 8  -columnspan 2 -sticky ew
	grid .frameMain.entryBlackHoleRepeatZ -in .frameMain.frameWarp	-row $currentRow -column 10 
}

proc setWarpCrackle {} {
   global activeWarp lastWarpRow
   set activeWarp "crackle"

   setWarpPriority
   eraseWarpControls

   # Left
   set currentRow 2
	grid .frameMain.labelCrackleForm1 -in .frameMain.frameWarp	-row $currentRow -column 1 -columnspan 2 -sticky e
	grid .frameMain.scaleCrackleForm1 -in .frameMain.frameWarp	-row $currentRow -column 3 -columnspan 2 -sticky ew
	grid .frameMain.entryCrackleForm1 -in .frameMain.frameWarp	-row $currentRow -column 5 

   set currentRow [ expr $currentRow + 1 ]
	grid .frameMain.labelCrackleForm2 -in .frameMain.frameWarp	-row $currentRow -column 1 -columnspan 2 -sticky e
	grid .frameMain.scaleCrackleForm2 -in .frameMain.frameWarp	-row $currentRow -column 3 -columnspan 2 -sticky ew
	grid .frameMain.entryCrackleForm2 -in .frameMain.frameWarp	-row $currentRow -column 5 

   set currentRow [ expr $currentRow + 1 ]
	grid .frameMain.labelCrackleForm3 -in .frameMain.frameWarp	-row $currentRow -column 1 -columnspan 2 -sticky e
	grid .frameMain.scaleCrackleForm3 -in .frameMain.frameWarp	-row $currentRow -column 3  -columnspan 2 -sticky ew
	grid .frameMain.entryCrackleForm3 -in .frameMain.frameWarp	-row $currentRow -column 5 

   set currentRow [ expr $currentRow + 1 ]
	grid .frameMain.cbCrackleSolid -in .frameMain.frameWarp	-row $currentRow -column 1 -columnspan 2 -sticky e

   set currentRow [ expr $currentRow + 1 ]
	grid .frameMain.labelCrackleMetric -in .frameMain.frameWarp	-row $currentRow -column 1 -columnspan 2 -sticky e
	grid .frameMain.scaleCrackleMetric -in .frameMain.frameWarp	-row $currentRow -column 3 -columnspan 2 -sticky ew
	grid .frameMain.entryCrackleMetric -in .frameMain.frameWarp	-row $currentRow -column 5 

   set currentRow [ expr $currentRow + 1 ]
	grid .frameMain.labelCrackleOffset -in .frameMain.frameWarp	-row $currentRow -column 1 -columnspan 2 -sticky e
	grid .frameMain.scaleCrackleOffset -in .frameMain.frameWarp	-row $currentRow -column 3 -columnspan 2 -sticky ew
	grid .frameMain.entryCrackleOffset -in .frameMain.frameWarp	-row $currentRow -column 5 

   # Right
   set currentRow 2 
   fillWarpBlanks $currentRow $lastWarpRow 6
}

proc setWarpCylindrical {} {
   global activeWarp lastWarpRow
   set activeWarp "cylindrical"

   setWarpPriority
   eraseWarpControls

   # Left
   set currentRow 2
	grid .frameMain.labelCSTOrientation -in .frameMain.frameWarp	-row $currentRow -column 1	-columnspan 2 -sticky e
	grid .frameMain.cbCSTOrientationX -in .frameMain.frameWarp	-row $currentRow -column 3	-sticky ew
	grid .frameMain.cbCSTOrientationY -in .frameMain.frameWarp	-row $currentRow -column 4	-sticky ew
	grid .frameMain.cbCSTOrientationZ -in .frameMain.frameWarp	-row $currentRow -column 5	-sticky ew

   set currentRow [ expr $currentRow + 1 ]
	grid .frameMain.labelCSTDistanceExp -in .frameMain.frameWarp	-row $currentRow -column 1 -columnspan 2 -sticky e
	grid .frameMain.scaleCSTDistanceExp -in .frameMain.frameWarp	-row $currentRow -column 3 -columnspan 2 -sticky ew
	grid .frameMain.entryCSTDistanceExp -in .frameMain.frameWarp	-row $currentRow -column 5 

   set currentRow [ expr $currentRow + 1 ]
   fillWarpBlanks $currentRow $lastWarpRow 6

   # Right
   set currentRow 2
   fillWarpBlanks $currentRow $lastWarpRow 1
}

proc setWarpSpherical {} {
   global activeWarp lastWarpRow
   set activeWarp "spherical"
   set topRow  2
   set nRows   3

   setWarpPriority
   eraseWarpControls

   # Left
   set currentRow 2
	grid .frameMain.labelCSTOrientation -in .frameMain.frameWarp	-row $currentRow -column 1	-columnspan 2 -sticky e
	grid .frameMain.cbCSTOrientationX -in .frameMain.frameWarp	-row $currentRow -column 3	-sticky ew
	grid .frameMain.cbCSTOrientationY -in .frameMain.frameWarp	-row $currentRow -column 4	-sticky ew
	grid .frameMain.cbCSTOrientationZ -in .frameMain.frameWarp	-row $currentRow -column 5	-sticky ew

   set currentRow [ expr $currentRow + 1 ]
	grid .frameMain.labelCSTDistanceExp -in .frameMain.frameWarp	-row $currentRow -column 1 -columnspan 2 -sticky e
	grid .frameMain.scaleCSTDistanceExp -in .frameMain.frameWarp	-row $currentRow -column 3 -columnspan 2 -sticky ew
	grid .frameMain.entryCSTDistanceExp -in .frameMain.frameWarp	-row $currentRow -column 5 

   set currentRow [ expr $currentRow + 1 ]
   fillWarpBlanks $currentRow $lastWarpRow 6

   # Right
   set currentRow 2
   fillWarpBlanks $currentRow $lastWarpRow 1
}

proc setWarpToroidal {} {
   global activeWarp lastWarpRow
   set activeWarp "toroidal"

   setWarpPriority
   eraseWarpControls

   # Left
   set currentRow 2
	grid .frameMain.labelCSTOrientation -in .frameMain.frameWarp	-row $currentRow -column 1	-columnspan 2 -sticky e
	grid .frameMain.cbCSTOrientationX -in .frameMain.frameWarp	-row $currentRow -column 3	-sticky ew
	grid .frameMain.cbCSTOrientationY -in .frameMain.frameWarp	-row $currentRow -column 4	-sticky ew
	grid .frameMain.cbCSTOrientationZ -in .frameMain.frameWarp	-row $currentRow -column 5	-sticky ew

   set currentRow [ expr $currentRow + 1 ]
	grid .frameMain.labelCSTDistanceExp -in .frameMain.frameWarp	-row $currentRow -column 1 -columnspan 2 -sticky e
	grid .frameMain.scaleCSTDistanceExp -in .frameMain.frameWarp	-row $currentRow -column 3 -columnspan 2 -sticky ew
	grid .frameMain.entryCSTDistanceExp -in .frameMain.frameWarp	-row $currentRow -column 5 

   set currentRow [ expr $currentRow + 1 ]
	grid .frameMain.labelToroidalRadius -in .frameMain.frameWarp	-row $currentRow -column 1 -columnspan 2 -sticky e
	grid .frameMain.scaleToroidalRadius -in .frameMain.frameWarp	-row $currentRow -column 3 -columnspan 2 -sticky ew
	grid .frameMain.entryToroidalRadius -in .frameMain.frameWarp	-row $currentRow -column 5 

   set currentRow [ expr $currentRow + 1 ]
   fillWarpBlanks $currentRow $lastWarpRow 1

   # Right
   set currentRow 2
   fillWarpBlanks $currentRow $lastWarpRow 6
}

proc setWarpPlanar {} {
   global activeWarp lastWarpRow
   set activeWarp "planar"

   setWarpPriority
   eraseWarpControls

   # Left
   set currentRow 2
	grid .frameMain.labelPlanarNormalX -in .frameMain.frameWarp	-row $currentRow -column 1 -columnspan 2 -sticky e
	grid .frameMain.scalePlanarNormalX -in .frameMain.frameWarp	-row $currentRow -column 3 -columnspan 2 -sticky ew
	grid .frameMain.entryPlanarNormalX -in .frameMain.frameWarp	-row $currentRow -column 5 

   set currentRow [ expr $currentRow + 1 ]
	grid .frameMain.labelPlanarNormalY -in .frameMain.frameWarp	-row $currentRow -column 1 -columnspan 2 -sticky e
	grid .frameMain.scalePlanarNormalY -in .frameMain.frameWarp	-row $currentRow -column 3 -columnspan 2 -sticky ew
	grid .frameMain.entryPlanarNormalY -in .frameMain.frameWarp	-row $currentRow -column 5 

   set currentRow [ expr $currentRow + 1 ]
	grid .frameMain.labelPlanarNormalZ -in .frameMain.frameWarp	-row $currentRow -column 1 -columnspan 2 -sticky e
	grid .frameMain.scalePlanarNormalZ -in .frameMain.frameWarp	-row $currentRow -column 3 -columnspan 2 -sticky ew
	grid .frameMain.entryPlanarNormalZ -in .frameMain.frameWarp	-row $currentRow -column 5 

   set currentRow [ expr $currentRow + 1  ]
	grid .frameMain.labelPlanarDistance -in .frameMain.frameWarp	-row $currentRow -column 1 -columnspan 2 -sticky e
	grid .frameMain.scalePlanarDistance -in .frameMain.frameWarp	-row $currentRow -column 3 -columnspan 2 -sticky ew
	grid .frameMain.entryPlanarDistance -in .frameMain.frameWarp	-row $currentRow -column 5 

   # Right
   set currentRow 2
   fillWarpBlanks $currentRow $lastWarpRow 6
}

proc setWarpRepeat {} {
   global activeWarp lastWarpRow
   set activeWarp "repeat"
   set topRow  2
   set nRows   3

   setWarpPriority
   eraseWarpControls

   # Left
   set currentRow 2
	grid .frameMain.labelRepeatDirection -in .frameMain.frameWarp	-row $currentRow -column 1	-columnspan 2 -sticky e
	grid .frameMain.cbRepeatDirectionX -in .frameMain.frameWarp	-row $currentRow -column 3	-sticky ew
	grid .frameMain.cbRepeatDirectionY -in .frameMain.frameWarp	-row $currentRow -column 4	-sticky ew
	grid .frameMain.cbRepeatDirectionZ -in .frameMain.frameWarp	-row $currentRow -column 5	-sticky ew

   set currentRow [ expr $currentRow + 1 ]
	grid .frameMain.labelRepeatLength -in .frameMain.frameWarp	-row $currentRow -column 1 -columnspan 2 -sticky e
	grid .frameMain.scaleRepeatLength -in .frameMain.frameWarp	-row $currentRow -column 3  -columnspan 2 -sticky ew
	grid .frameMain.entryRepeatLength -in .frameMain.frameWarp	-row $currentRow -column 5 

   set currentRow [ expr $currentRow + 1 ]
	grid .frameMain.labelRepeatFlip -in .frameMain.frameWarp	-row $currentRow -column 1	-columnspan 2 -sticky e
	grid .frameMain.cbRepeatFlipX -in .frameMain.frameWarp	-row $currentRow -column 3	-sticky ew
	grid .frameMain.cbRepeatFlipY -in .frameMain.frameWarp	-row $currentRow -column 4	-sticky ew
	grid .frameMain.cbRepeatFlipZ -in .frameMain.frameWarp	-row $currentRow -column 5	-sticky ew

   set currentRow [ expr $currentRow + 1 ]
   fillWarpBlanks $currentRow $lastWarpRow 1

   # Right
   set currentRow 2
	grid .frameMain.labelRepeatOffsetX -in .frameMain.frameWarp	-row $currentRow -column 6 -columnspan 2 -sticky e
	grid .frameMain.scaleRepeatOffsetX -in .frameMain.frameWarp	-row $currentRow -column 8 -columnspan 2 -sticky ew
	grid .frameMain.entryRepeatOffsetX -in .frameMain.frameWarp	-row $currentRow -column 10 

   set currentRow [ expr $currentRow + 1 ]
	grid .frameMain.labelRepeatOffsetY -in .frameMain.frameWarp	-row $currentRow -column 6 -columnspan 2 -sticky e
	grid .frameMain.scaleRepeatOffsetY -in .frameMain.frameWarp	-row $currentRow -column 8 -columnspan 2 -sticky ew
	grid .frameMain.entryRepeatOffsetY -in .frameMain.frameWarp	-row $currentRow -column 10 

   set currentRow [ expr $currentRow + 1 ]
	grid .frameMain.labelRepeatOffsetZ -in .frameMain.frameWarp	-row $currentRow -column 6 -columnspan 2 -sticky e
	grid .frameMain.scaleRepeatOffsetZ -in .frameMain.frameWarp	-row $currentRow -column 8 -columnspan 2 -sticky ew
	grid .frameMain.entryRepeatOffsetZ -in .frameMain.frameWarp	-row $currentRow -column 10 

   set currentRow [ expr $currentRow + 1 ]
   fillWarpBlanks $currentRow $lastWarpRow 6
}

proc setWarpTurbulence {} {
   global activeWarp lastWarpRow
   set activeWarp "turbulence"

   setWarpPriority
   eraseWarpControls

   # Left
   set currentRow 2
	grid .frameMain.labelTurbulenceX -in .frameMain.frameWarp	-row $currentRow -column 1 -columnspan 2 -sticky e
	grid .frameMain.scaleTurbulenceX -in .frameMain.frameWarp	-row $currentRow -column 3 -columnspan 2 -sticky ew
	grid .frameMain.entryTurbulenceX -in .frameMain.frameWarp	-row $currentRow -column 5 

   set currentRow [ expr $currentRow + 1 ]
	grid .frameMain.cbLockTurbulenceY -in .frameMain.frameWarp	-row $currentRow -column 1 -columnspan 2 -sticky e
	grid .frameMain.scaleTurbulenceY -in .frameMain.frameWarp	-row $currentRow -column 3 -columnspan 2 -sticky ew
	grid .frameMain.entryTurbulenceY -in .frameMain.frameWarp	-row $currentRow -column 5 

   set currentRow [ expr $currentRow + 1 ]
	grid .frameMain.labelTurbulenceZ -in .frameMain.frameWarp	-row $currentRow -column 1 -columnspan 2 -sticky e
	grid .frameMain.scaleTurbulenceZ -in .frameMain.frameWarp	-row $currentRow -column 3 -columnspan 2 -sticky ew
	grid .frameMain.entryTurbulenceZ -in .frameMain.frameWarp	-row $currentRow -column 5 

   set currentRow [ expr $currentRow + 1 ]
   fillWarpBlanks $currentRow $lastWarpRow 1

   # Right
   set currentRow 2
	grid .frameMain.labelOctaves -in .frameMain.frameWarp	-row $currentRow -column 6 -columnspan 2 -sticky e
	grid .frameMain.scaleOctaves -in .frameMain.frameWarp	-row $currentRow -column 8  -columnspan 2 -sticky ew
	grid .frameMain.entryOctaves -in .frameMain.frameWarp	-row $currentRow -column 10 

   set currentRow [ expr $currentRow + 1 ]
	grid .frameMain.labelLambda -in .frameMain.frameWarp	-row $currentRow -column 6 -columnspan 2 -sticky e
	grid .frameMain.scaleLambda -in .frameMain.frameWarp	-row $currentRow -column 8  -columnspan 2 -sticky ew
	grid .frameMain.entryLambda -in .frameMain.frameWarp	-row $currentRow -column 10 

   set currentRow [ expr $currentRow + 1 ]
	grid .frameMain.labelOmega -in .frameMain.frameWarp	-row $currentRow -column 6 -columnspan 2 -sticky e
	grid .frameMain.scaleOmega -in .frameMain.frameWarp	-row $currentRow -column 8  -columnspan 2 -sticky ew
	grid .frameMain.entryOmega -in .frameMain.frameWarp	-row $currentRow -column 10 

   set currentRow [ expr $currentRow + 1 ]
   fillWarpBlanks $currentRow $lastWarpRow 6
}

# Set interior type and associated variables,
# i. e. Index Of Refraction (IOR).
proc setAir {} {
   global varIor varInterior
   set varIor 1.00
   set varInterior "air"
}
proc setWater {} {
   global varIor varInterior
   set varIor 1.33
   set varInterior "water"
}
proc setGlass {} {
   global varIor varInterior
   set varIor 1.50
   set varInterior "glass"
}
proc setDiamond {} {
   global varIor varInterior
   set varIor 2.40
   set varInterior "diamond"
}

# Calculate widget and element selected.
# Pattern selection and associated variable updates,
proc updatePattern { widget n } {
   global activePattern 

#   set nColors 1

   set activePattern [$widget get $n\n]
#   puts "$activePattern\n"
   setPattern
}

proc setPattern { } {
   global activePattern varRed varGreen varBlue 
   global stateColormap stateGradient
   global varColormapN nColors varLabelColormapN activeColor nColorsAdded
   global varEntryColormapN listColormapRed listColormapGreen listColormapBlue

   set stateGradient 0
   disableGradient
   set stateColormap 0
   disableColormap
#   set activeColor 0
#   set varColormapN 0.00

   # Non-colormap patterns, each has a set number of colors.
   switch $activePattern {
      "brick"     { set nColors 2 }
      "checker"   { set nColors 2 }
      "color"     { set nColors 1 }
      "hexagon"   { set nColors 3 }
      "image map"  { set nColors 0 }
      "object"    { set nColors 0 }

      default     {
         set stateColormap 1
         enableColormap
      }
   }
   
   # Colormap is now enabled, if no previous pattern was chosen.
   if { $stateColormap == 1 } {

      if { $nColors < 2 } {
         set nColors 2
      }

      set nColorsAdded [ expr $nColors - 2 ]

      switch $activePattern {
         "agate"     { }
         "bozo"      { }
         "crackle"   { }
         "gradient"  {
            set stateGradient 1
            enableGradient
         }
         "granite"   { }
         "julia"     { }
         "leopard"   { }
         "mandel"    { }
         "marble"    { }
         "radial"    { }
         "spotted"   { }
         "waves"     { }
         "wood"      { }
         "wrinkles"  { }

         default { }
      }
   }

   # Truncate displays for limited-color patterns
    updateLabelColormapN $nColors
    if { $varEntryColormapN > $nColors } {

      set varEntryColormapN $nColors
      updateEntryColormapN  $nColors

      # Temporary patch, actually a special display is required for zero-color textures!
#      if { $nColors == 0 } {
#         set nColors 1
#      }

      if { $nColors != 0 } {
         set varRed     [ lindex $listColormapRed    [ expr $nColors - 1 ] ] 
         set varGreen   [ lindex $listColormapGreen  [ expr $nColors - 1 ] ]
         set varBlue    [ lindex $listColormapBlue   [ expr $nColors - 1 ] ]
      }
      updateRGBCanvas
   }
   updateColormapN $varColormapN 
   updateColormapCanvas
}

proc updateFade { varFadeRGB } {
   global varFadeLock varFadeRed varFadeGreen varFadeBlue

   if { $varFadeLock == 1 } {
      set varFadeRed   $varFadeRGB
      set varFadeGreen $varFadeRGB
      set varFadeBlue  $varFadeRGB
   }
}

# Pattern image map controls.
proc updateInterpBilinear {} {
   global varInterpBilinear varInterpNorm

   if { $varInterpBilinear == 1 } {
      set varInterpNorm 0
   }
}

proc updateInterpNorm {} {
   global varInterpBilinear varInterpNorm

   if { $varInterpNorm == 1 } {
      set varInterpBilinear 0
   }
}

proc updateImageMapPlanar {} {
   global varImageMapPlanar varImageMapSphere varImageMapCylinder varImageMapTorus
   set varImageMapSphere   0
   set varImageMapCylinder 0
   set varImageMapTorus    0

   if { $varImageMapPlanar == 0 } {
      set varImageMapPlanar 1
   }
}

proc updateImageMapSphere {} {
   global varImageMapPlanar varImageMapSphere varImageMapCylinder varImageMapTorus
   global varImageMapOnceTemp varImageMapOnce

   set varImageMapPlanar   0
   set varImageMapCylinder 0
   set varImageMapTorus    0

   if { $varImageMapSphere == 0 } {
      set varImageMapPlanar 1
   }
}

proc updateImageMapCylinder {} {
   global varImageMapPlanar varImageMapSphere varImageMapCylinder varImageMapTorus
   set varImageMapPlanar   0
   set varImageMapSphere   0
   set varImageMapTorus    0

   if { $varImageMapCylinder == 0 } {
      set varImageMapPlanar 1
   }
}

proc updateImageMapTorus {} {
   global varImageMapPlanar varImageMapSphere varImageMapCylinder varImageMapTorus
   set varImageMapPlanar   0
   set varImageMapSphere   0
   set varImageMapCylinder 0

   if { $varImageMapTorus == 0 } {
      set varImageMapPlanar 1
   }
}

###############
# File I/O menu

set fileInc " "
set filePOV "tkxture.inc"
set bufferTexture "..."

# gif | tga | iff | ppm | pgm | png | jpeg | tiff | sys
set imageTypes {
   {{Image files} {.gif .tga .iff .ppm .pgm .png .jpeg .tiff .sys}  }
   {{All Files}        *  }
}

proc imageOpen {} {
   global fileArchive imageTypes activeImage
   set fileArchive [tk_getOpenFile -filetypes $imageTypes]
   # bail out if operation is cancelled 
   if { $fileArchive != "" } {
      set activeImage $fileArchive
   }
}

# POV-Ray Include files.

set fileTypes {
   {{Include files} {.inc} TEXT }
   {{All Files}        *  }
}

proc fileExit {} {
   exit
}

proc fileNew {} {
}

proc fileOpen {} {
   global fileArchive fileTypes 
   set fileArchive [tk_getOpenFile -filetypes $fileTypes]
   # bail out if operation is cancelled 
   if { $fileArchive != "" } {

   }
}

proc fileSave {} {
   global fileTypes fileInc fileArchive activeFile
   set fileArchive [ tk_getSaveFile -defaultextension inc -initialfile $fileArchive -filetypes $fileTypes -title "Save" ]
   # bail out if operation is cancelled
   if { $fileArchive != "" } {
      set activeFile $fileArchive
      writePOV $fileArchive
   }
}

proc fileSaveAs {} {
   global fileTypes fileInc fileArchive activeFile
   set fileArchive [ tk_getSaveFile -defaultextension inc -initialfile $fileArchive -filetypes $fileTypes -title "Save As" ]
   # bail out if operation is cancelled 
   if { $fileArchive != "" } {   
      set activeFile $fileArchive
      writePOV $fileArchive
   }  
}

# Write POV-Ray file prior to rendering.
proc writePOVtmp { varFile } {

   global bufferTexture activeFile activeTexture
   global activePattern stateColormap
#   puts $varFile

   # tell buffTexture the filename
   set activeFile $varFile
   # buffer the POV-Ray texture definition.
   buffTexture "Temp"

   set bufferPOV $bufferTexture

   # Open file for write, write file and close.
   set f [ open $varFile w ]
   puts $f $bufferPOV
   close $f
}

# This file may  be saved as a named texture file as well.
proc writePOV { varFile } {

   global bufferTexture
   global activeTexture activePattern stateColormap
#   puts $varFile

   # tell buffTexture the filename
   set activeFile $varFile
   # buffer the POV-Ray texture definition.
   buffTexture $activeTexture

   set bufferPOV $bufferTexture

   # Open file for write, write file and close.
   set f [ open $varFile w ]
   puts $f $bufferPOV
   close $f
}

###########
# Edit menu

###########
# Help menu

# TBD figure out how to modify OS environment.

proc helpCommand {} {

   exec firefox "http://www.rgbterrain.com/help/tkxture/" & 
}

proc helpAbout {} {
}

# Hitting Enter (Return) releases input focus from entry widget back to the toplevel frame.
proc setEntryState {} {
   focus .frameMain
}


############
### Main ###

# Alt-r or Alt-R to render from anywhere in the application.
bind . <Alt-KeyPress-r> { execRender }
bind . <Alt-KeyPress-R> { execRender }
bind . <KeyPress-Return> { setEntryState }

# Update colormap display after scale-thumb is released.
# bind .frameMain.frameColormap.scaleRed            <ButtonRelease-1> { updateColormapCanvas }
# bind .frameMain.frameColormap.scaleGreen          <ButtonRelease-1> { updateColormapCanvas }
# bind .frameMain.frameColormap.scaleBlue           <ButtonRelease-1> { updateColormapCanvas }
# bind .frameMain.frameColormap.scaleColormapLevel  <ButtonRelease-1> { updateColormapCanvas }

bind .frameMain.scaleRed            <ButtonRelease-1> { updateColormapCanvas }
bind .frameMain.scaleGreen          <ButtonRelease-1> { updateColormapCanvas }
bind .frameMain.scaleBlue           <ButtonRelease-1> { updateColormapCanvas }
bind .frameMain.scaleColormapLevel  <ButtonRelease-1> { updateColormapCanvas }

setAgate
setWarpTurbulence

initColormap
execRender

# end tkxture.tcl
